@@ -2,7 +2,7 @@
 
   Program: GDCM (Grassroots DICOM). A DICOM library
 
-  Copyright (c) 2006-2011 Mathieu Malaterre
+  Copyright (c) 2006-2016 Mathieu Malaterre
   All rights reserved.
   See Copyright.txt or http://gdcm.sourceforge.net/Copyright.html for details.
 
@@ -22,7 +22,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <string.h>
+#include <string.h> // strspn
 #include <assert.h>
 #include <errno.h>
 #include <sys/stat.h>
@@ -42,18 +42,16 @@
 #elif defined(GDCM_HAVE__SNPRINTF)
 #define snprintf _snprintf
 #endif
-#ifdef __APPLE__
-#include <CoreFoundation/CFBase.h>
-#include <CoreFoundation/CFBundle.h>
-#include <CoreFoundation/CFURL.h>
-#endif // __APPLE__
+#ifdef GDCM_USE_COREFOUNDATION_LIBRARY
+#include <CoreFoundation/CoreFoundation.h>
+#endif
 
 #if defined(_WIN32) && (defined(_MSC_VER) || defined(__WATCOMC__) ||defined(__BORLANDC__) || defined(__MINGW32__))
 #include <io.h>
 #include <direct.h>
 #define _unlink unlink
 #else
-//#include <features.h>	// we want GNU extensions
+//#include <features.h> // we want GNU extensions
 #include <dlfcn.h>
 #include <sys/types.h>
 #include <fcntl.h>
@@ -142,6 +140,8 @@ const char * System::GetCWD()
 
 bool System::MakeDirectory(const char *path)
 {
+  if( !path || !*path )
+    return false;
   if(System::FileExists(path))
     {
     return true;
@@ -155,12 +155,14 @@ bool System::MakeDirectory(const char *path)
     pos = 0;
     }
   std::string topdir;
-  while((pos = dir.find('/', pos)) != std::string::npos)
+  bool ok = true;
+  while(ok && (pos = dir.find('/', pos)) != std::string::npos)
     {
-    topdir = dir.substr(0, pos);
-    Mkdir(topdir.c_str());
+    topdir = dir.substr(0, pos+1);
+    ok = ok && (System::FileIsDirectory(topdir.c_str()) || 0 == Mkdir(topdir.c_str()));
     pos++;
     }
+  if( !ok ) return false;
   if(dir[dir.size()-1] == '/')
     {
     topdir = dir.substr(0, dir.size());
@@ -279,7 +281,7 @@ bool System::GetPermissions(const char* file, unsigned short& mode)
     {
     return false;
     }
-  mode = st.st_mode;
+  mode = (short)st.st_mode;
   return true;
 }
 
@@ -363,6 +365,10 @@ bool System::DeleteDirectory(const char *source)
   return Rmdir(source) == 0;
 }
 
+#ifndef PATH_MAX
+#define PATH_MAX 4096
+#endif
+
 // return size of file; also returns zero if no file exists
 size_t System::FileSize(const char* filename)
 {
@@ -398,51 +404,6 @@ size_t System::FileSize(const char* filename)
   return size2;
 }
 
-#if 0
-const char *System::GetCurrentDataDirectory()
-{
-#ifdef _WIN32
-  static char path[MAX_PATH];
-  gdcm::Filename fn( GetCurrentProcessFileName() );
-  if ( !fn.IsEmpty() )
-    {
-    std::string str = fn.GetPath();
-    str += "/../" GDCM_INSTALL_DATA_DIR;
-    strcpy(path, str.c_str());
-    return path;
-    }
-#else
-
-  static char path[PATH_MAX];
-
-#ifdef __APPLE__
-  Boolean success = false;
-  CFURLRef pathURL = CFBundleCopyResourcesDirectoryURL(CFBundleGetMainBundle());
-  if (pathURL != NULL)
-    {
-    success = CFURLGetFileSystemRepresentation(pathURL, true /*resolveAgainstBase*/, (unsigned char*) path, PATH_MAX);
-    CFRelease(pathURL);
-    }
-  if (success)
-    {
-    strncat(path, "/" GDCM_INSTALL_DATA_DIR, PATH_MAX);
-    return path;
-    }
-#endif
-
-  gdcm::Filename fn( GetCurrentProcessFileName() );
-  if ( !fn.IsEmpty() )
-    {
-    std::string str = fn.GetPath();
-    str += "/../" GDCM_INSTALL_DATA_DIR;
-    strcpy(path, str.c_str());
-    return path;
-    }
-#endif
-  return 0;
-}
-#endif
-
 /*
  * TODO:
  * check cygwin
@@ -450,9 +411,6 @@ const char *System::GetCurrentDataDirectory()
  * check solaris
  * check hpux
  * check os2: DosGetInfoBlocks / DosQueryModuleName
- * check macosx :
- *  ProcessSerialNumber psn = {kNoProcess, kCurrentProcess};
- *  GetProcessInformation -> FSMakeFSSpec
  * ...
  */
 const char *System::GetCurrentProcessFileName()
@@ -463,7 +421,7 @@ const char *System::GetCurrentProcessFileName()
     {
     return buf;
     }
-#elif defined(__APPLE__)
+#elif defined(GDCM_USE_COREFOUNDATION_LIBRARY)
   static char buf[PATH_MAX];
   Boolean success = false;
   CFURLRef pathURL = CFBundleCopyExecutableURL(CFBundleGetMainBundle());
@@ -476,15 +434,33 @@ const char *System::GetCurrentProcessFileName()
     {
     return buf;
     }
-#else
+#elif defined (__SVR4) && defined (__sun)
+  // solaris
+  const char *ret = getexecname();
+  if( ret ) return ret;
+//#elif defined(__NetBSD__)
+//  static char path[PATH_MAX];
+//  if ( readlink ("/proc/curproc/exe", path, sizeof(path)) > 0)
+//    {
+//    return path;
+//    }
+#elif defined(__DragonFly__) || defined(__OpenBSD__) || defined(__FreeBSD__)
+  static char path[PATH_MAX];
+  if ( readlink ("/proc/curproc/file", path, sizeof(path)) > 0)
+    {
+    return path;
+    }
+#elif defined(__linux__)
   static char path[PATH_MAX];
   if ( readlink ("/proc/self/exe", path, sizeof(path)) > 0) // Technically 0 is not an error, but that would mean
                                                             // 0 byte were copied ... thus considered it as an error
     {
     return path;
     }
+#else
+  gdcmErrorMacro( "missing implementation" );
 #endif
-   return 0;
+   return nullptr;
 }
 
 #ifdef __USE_GNU
@@ -509,28 +485,28 @@ const char *System::GetCurrentModuleFileName()
   return System::GetCurrentProcessFileName();
 #endif
 
-  return 0;
+  return nullptr;
 }
 
 const char *System::GetCurrentResourcesDirectory()
 {
-#ifdef __APPLE__
+#ifdef GDCM_USE_COREFOUNDATION_LIBRARY
   static char path[PATH_MAX];
   Boolean success = false;
   CFURLRef pathURL = CFBundleCopyResourcesDirectoryURL(CFBundleGetMainBundle());
-  if (pathURL != NULL)
+  if (pathURL != nullptr)
     {
     success = CFURLGetFileSystemRepresentation(pathURL, true /*resolveAgainstBase*/, (unsigned char*) path, PATH_MAX);
     CFRelease(pathURL);
     }
   if (success)
     {
-    strncat(path, "/" GDCM_INSTALL_DATA_DIR, PATH_MAX);
+    strlcat(path, "/" GDCM_INSTALL_DATA_DIR, PATH_MAX);
     return path;
     }
 #endif
   // Is there such beast on *any* other system but APPLE ?
-  return 0;
+  return nullptr;
 }
 
 /**
@@ -543,7 +519,7 @@ inline int getlastdigit(unsigned char *data, unsigned long size)
   for(unsigned int i=0;i<size;i++)
     {
     extended = (carry << 8) + data[i];
-    data[i] = extended / 10;
+    data[i] = (unsigned char)(extended / 10);
     carry = extended % 10;
     }
   assert( carry >= 0 && carry < 10 );
@@ -561,7 +537,7 @@ size_t System::EncodeBytes(char *out, const unsigned char *data, int size)
   while(!zero)
     {
     res = getlastdigit(addr, size);
-    const char v = '0' + res;
+    const char v = (char)('0' + res);
     sres.insert(sres.begin(), v);
     zero = true;
     for(int i = 0; i < size; ++i)
@@ -703,6 +679,7 @@ bool System::ParseDateTime(time_t &timep, long &milliseconds, const char date[22
     case 3: hour = 0;
     case 4: min = 0;
     case 5: sec = 0;
+      break; // http://security.coverity.com/blog/2013/Sep/gimme-a-break.html
       }
     ptm.tm_year = year - 1900;
     if( mon < 1 || mon > 12 ) return false;
@@ -748,7 +725,7 @@ const char *System::GetTimezoneOffsetFromUTC()
 {
   static std::string buffer;
   char outstr[10];
-  time_t t = time(NULL);
+  time_t t = time(nullptr);
   struct tm *tmp = localtime(&t);
   size_t l = strftime(outstr, sizeof(outstr), "%z", tmp);
   assert( l == 5 ); (void)l;
@@ -759,15 +736,24 @@ const char *System::GetTimezoneOffsetFromUTC()
 bool System::FormatDateTime(char date[22], time_t timep, long milliseconds)
 {
   // \precondition
-  if( !(milliseconds >= 0 && milliseconds < 1000000) ) return false;
+  if( !(milliseconds >= 0 && milliseconds < 1000000) )
+    {
+    return false;
+    }
 
   // YYYYMMDDHHMMSS.FFFFFF&ZZXX
-  if(!date) return false;
+  if(!date)
+    {
+    return false;
+    }
   const size_t maxsize = 40;
   char tmp[maxsize];
   // Obtain the time of day, and convert it to a tm struct.
   struct tm *ptm = localtime (&timep);
-  if(!ptm) return false;
+  if(!ptm)
+    {
+    return false;
+    }
   // Format the date and time, down to a single second.
   size_t ret = strftime (tmp, sizeof (tmp), "%Y%m%d%H%M%S", ptm);
   assert( ret == 14 );
@@ -778,10 +764,9 @@ bool System::FormatDateTime(char date[22], time_t timep, long milliseconds)
 
   // Add milliseconds
   const size_t maxsizall = 22;
-  //char tmpAll[maxsizall];
-  int ret2 = snprintf(date,maxsizall,"%s.%06ld",tmp,milliseconds);
-  assert( ret2 >= 0 );
-  if( (unsigned int)ret2 >= maxsizall )
+  const int ret2 = snprintf(date,maxsizall,"%s.%06ld",tmp,milliseconds);
+  if( ret2 < 0 ) return false;
+  if( (size_t)ret2 >= maxsizall )
     {
     return false;
     }
@@ -827,7 +812,7 @@ bool System::GetCurrentDateTime(char date[22])
   // Apparently suseconds_t is defined as long on linux system... why would this be signed ?
 
   struct timeval tv;
-  gettimeofday (&tv, NULL);
+  gettimeofday (&tv, nullptr);
   timep = tv.tv_sec;
   // A concatenated date-time character string in the format:
   // YYYYMMDDHHMMSS.FFFFFF&ZZXX
@@ -914,40 +899,56 @@ bool System::GetHostName(char name[255])
   return false;
 }
 
-char *System::StrTokR(char *ptr, const char *sep, char **end)
+char *System::StrTokR(char *str, const char *delim, char **nextp)
 {
 #if 1
-  if (ptr == NULL)
+  // http://groups.google.com/group/comp.lang.c/msg/2ab1ecbb86646684
+  // PD -> http://groups.google.com/group/comp.lang.c/msg/7c7b39328fefab9c
+  char *ret;
+
+  if (str == nullptr)
     {
-    ptr = *end;
+    str = *nextp;
     }
 
-  /* search string for set of char: strspn */
-  while (*ptr && strchr(sep, *ptr))
+  str += strspn(str, delim);
+
+  if (*str == '\0')
     {
-    ++ptr;
+    return nullptr;
     }
 
-  if (*ptr == '\0') return NULL;
+  ret = str;
 
-  char *token = ptr;
-  *end = token + 1;
+  str += strcspn(str, delim);
 
-  char *pend = *end;
-  while (*pend && !strchr(sep, *pend))
+  if (*str)
     {
-    ++pend;
+    *str++ = '\0';
     }
 
-  if (*pend)
-    {
-    *pend = '\0';
-    ++pend;
-    }
+  *nextp = str;
 
-  return token;
+  return ret;
 #else
-  return strtok_r(ptr,sep,end);
+  return strtok_r(str,delim,nextp);
+#endif
+}
+
+char *System::StrSep(char **sp, const char *sep)
+{
+  // http://unixpapa.com/incnote/string.html
+  // http://stackoverflow.com/questions/8512958/is-there-a-windows-variant-of-strsep
+#if 1
+  char *p, *s;
+  if (sp == nullptr || *sp == nullptr || **sp == '\0') return nullptr;
+  s = *sp;
+  p = s + strcspn(s, sep);
+  if (*p != '\0') *p++ = '\0';
+  *sp = p;
+  return s;
+#else
+  return strsep(sp, sep);
 #endif
 }
 
@@ -960,9 +961,13 @@ struct CharsetAliasType
 #if defined(_WIN32)
 static const char *CharsetAliasToName(const char *alias)
 {
+  assert( alias );
+  //gdcmDebugMacro( alias );
+  // http://msdn.microsoft.com/en-us/library/windows/desktop/dd317756(v=vs.85).aspx
+  // 1252 windows-1252  ANSI Latin 1; Western European (Windows)
   static CharsetAliasType aliases[] = {
-    { "CP1252", "UTF-8" }, // mingw + debian/6.0
-    { NULL, NULL },
+      { "CP1252", "ISO-8859-1" }, // mingw + debian/6.0
+      { NULL, NULL },
   };
   for( CharsetAliasType *a = aliases; a->alias; a++)
     {
@@ -972,13 +977,14 @@ static const char *CharsetAliasToName(const char *alias)
       }
     }
   // We need to tell the user...
+  gdcmWarningMacro( std::string("Could not find Charset from alias: ") + alias );
   return NULL;
 }
 #endif //_WIN32
 
 const char *System::GetLocaleCharset()
 {
-  const char *codeset = NULL;
+  const char *codeset = nullptr;
 #if defined(GDCM_HAVE_NL_LANGINFO)
   //setlocale (LC_CTYPE, NULL);
   /* According to documentation nl_langinfo needs :
