@@ -242,6 +242,8 @@ static const char * const printmd5[][2] = {
 { "1daa9bb53a8ce090041472f528248686" , "IllegalGroup2ImplicitTS.dcm" },
 { "14c982413ba9bce288c2af88f4f751b6" , "GE_MR_0025xx1bProtocolDataBlockXML.dcm" },
 { "df3327313dadcc29e88779c7bc884abb" , "EmptyIcon_Bug417.dcm" },
+{ "ebd268e4cf018ac02a694d75f2cc93fd" , "JPEGLosslessSeNonZero.dcm" },
+{ "7c416c1dc36e3fa593ba029b94a900eb" , "US-YBR_FULL_422-EVRLE.dcm" },
 
 
 { nullptr ,nullptr }
